@@ -7,7 +7,7 @@ Built-in Digirule models supported by dgtools.
 """
 from .exceptions import DgtoolsErrorOpcodeNotSupported, DgtoolsErrorProgramHalt, DgtoolsErrorOutOfMemory
 from .callbacks import (DigiruleCallbackInputBase, DigiruleCallbackInputUserInteraction, DigiruleCallbackComOutStdout, 
-                        DigiruleCallbackComInUserInteraction)
+                        DigiruleCallbackComInUserInteraction, DigiruleCallbackPinInUserInteraction)
 import random
 import pyparsing
 
@@ -557,20 +557,29 @@ class Digirule2U(Digirule):
                                 47:self._randa,
                                192:self._comout,
                                193:self._comin,
-                               194:self._comrdy})
+                               194:self._comrdy,
+                               196:self._pinout,
+                               197:self._pinin,
+                               198:self._pindir})
                                
         self._comout_callback = None
         self._comin_callback = None
+        self._pin_in_callback = None
+        self._pin_out_callback = None
         
     def set_default_callbacks(self):
         super().set_default_callbacks()
-        self._comin_callback = DigiruleCallbackComInUserInteraction("Serial Input:")
-        self._comout_callback = DigiruleCallbackComOutStdout()
+        self._comin_callback = DigiruleCallbackComInUserInteraction("Serial Input <-")
+        self._pin_in_callback = DigiruleCallbackPinInUserInteraction("Pin Input <-")
+        self._comout_callback = DigiruleCallbackComOutStdout("Serial Output ->")
+        self._pin_out_callback = DigiruleCallbackComOutStdout("Pin Output ->")
         
     def clear_callbacks(self):
         super().clear_callbacks()
         self._comin_callback = None
         self._comout_callback = None
+        self._pin_in_callback = None
+        self._pin_out_callback = None
         
     @property 
     def comout_callback(self):
@@ -578,8 +587,8 @@ class Digirule2U(Digirule):
         
     @comout_callback.setter
     def comout_callback(self, new_callback):
-        if not isinstance(new_callback, DigiruleCallbackInputBase):
-            raise TypeError(f"comout_callback() setter expected descendant of DigiruleCallbackInputBase, "
+        if not isinstance(new_callback, DigiruleCallbackOutputBase):
+            raise TypeError(f"comout_callback() setter expected descendant of DigiruleCallbackOutputBase, "
                             f"received {type(new_callback)}")
         self._comout_callback = new_callback
         
@@ -589,10 +598,33 @@ class Digirule2U(Digirule):
         
     @comin_callback.setter
     def comin_callback(self, new_callback):
-        if type(new_callback) is not types.FunctionType:
-            raise TypeError(f"comin_callback() setter expected a function, received {type(new_callback)}")
+        if not isinstance(new_callback, DigiruleCallbackInputBase):
+            raise TypeError(f"comin_callback() setter expected descendant of DigiruleCallbackInputBase, "
+                            f"received {type(new_callback)}")
         self._comin_callback = new_callback
 
+    @property
+    def pin_in_callback(self):
+            return self._pin_in_callback
+            
+    @pin_in_callback.setter
+    def pin_in_callback(self, new_callback):
+        if not isinstance(new_callback, DigiruleCallbackInputBase):
+            raise TypeError(f"pin_in_callback() setter expected descendant of DigiruleCallbackInputBase, "
+                            f"received {type(new_callback)}")
+        self._pin_in_callback = new_callback
+    
+    @property
+    def pin_out_callback(self):
+        return self._pin_out_callback
+    
+    @pin_out_callback.setter
+    def pin_out_callback(self, new_callback):
+        if not isinstance(new_callback, DigiruleCallbackInputBase):
+            raise TypeError(f"comin_callback() setter expected descendant of DigiruleCallbackInputBase, "
+                            f"received {type(new_callback)}")
+        self._pin_out_callback = new_callback
+            
     def _initsp(self):
         self._ppc = []
         
@@ -737,6 +769,62 @@ class Digirule2U(Digirule):
         # Comms is always "ready" in emulation.
         # TODO: MED, Maybe this can be matched to a more realistic behaviour once comout, comin are connected to real files.
         self._set_status_reg(self._ZERO_FLAG_BIT, 0)
+        
+    # TODO: MID, Reduce code duplication in _comin, _comout
+    def _pinout(self):
+        # Send ACC n bit to n_pin.
+        # The n_pin is a mask
+        n_pin = self._read_next()
+        if n_pin>=1 and n_pin<=3:
+            if self._pin_out_callback is not None:
+                if n_pin != 3:
+                    # Only one pin is sent to the output
+                    pin_label = f"(PIN {n_pin}) "
+                    self._pin_out_callback.label=f"{pin_label}{self._pin_out_callback.label}"
+                    self._pin_out_callback(48 + ((self._get_acc_value() & n_pin)>>(n_pin-1)))
+                    self._pin_out_callback.label = self._pin_out_callback.label.replace(pin_label,"")
+                else:
+                    # Both pins are sent to the output
+                    pin_label = f"(PIN 1) "
+                    self._pin_out_callback.label=f"{pin_label}{self._pin_out_callback.label}"
+                    self._pin_out_callback(48 + (self._get_acc_value() & 1))
+                    self._pin_out_callback.label = self._pin_out_callback.label.replace(pin_label,"")
+                    
+                    pin_label = f"(PIN 2) "
+                    self._pin_out_callback.label=f"{pin_label}{self._pin_out_callback.label}"
+                    self._pin_out_callback(48 + ((self._get_acc_value() & 2)>>1))
+                    self._pin_out_callback.label = self._pin_out_callback.label.replace(pin_label,"")
+        
+    def _pinin(self):
+        # Read ACC n bit to n_pin (1-3)
+        # The n_pin is a mask if n_pin<3. If n_pin is 3 then the operation is carried out on both pins
+        n_pin = self._read_next()
+        if n_pin>=1 and n_pin<=3:
+            if self._pin_in_callback is not None:
+                if n_pin != 3:
+                    # Only one pin is read
+                    # TODO: LOW, Improve the way the label is modified externally
+                    pin_label = f"(PIN {n_pin}) "
+                    self._pin_in_callback.label=f"{pin_label}{self._pin_in_callback.label}"
+                    self._set_acc_value((self._get_acc_value() & ~n_pin) | self._pin_in_callback())
+                    self._pin_in_callback.label = self._pin_in_callback.label.replace(pin_label,"")
+                else:
+                    # Both pins are read
+                    pin_label = f"(PIN 1) "
+                    self._pin_in_callback.label=f"{pin_label}{self._pin_in_callback.label}"
+                    self._set_acc_value((self._get_acc_value() & ~1) | self._pin_in_callback())
+                    self._pin_in_callback.label = self._pin_in_callback.label.replace(pin_label,"")
+                    
+                    pin_label = f"(PIN 2) "
+                    self._pin_in_callback.label=f"{pin_label}{self._pin_in_callback.label}"
+                    self._set_acc_value((self._get_acc_value() & ~2) | (self._pin_in_callback()<<1))
+                    self._pin_in_callback.label = self._pin_in_callback.label.replace(pin_label,"")
+        
+    def _pindir(self):
+        # Set the pin I/O direction
+        # The n_pin is a mask
+        n_pin = self._read_next()
+        
 
     @staticmethod
     def get_asm_statement_def(existing_defs):
@@ -791,8 +879,8 @@ class Digirule2U(Digirule):
         asm_sbr = pyparsing.Group(pyparsing.Regex("SBR|BSET")("cmd") + existing_defs["literal_or_identifier"]("n_bit") + existing_defs["literal_or_identifier"]("addr"))("36:2")
         asm_bchg = pyparsing.Group(pyparsing.Regex("BCHG")("cmd") + existing_defs["literal_or_identifier"]("n_bit") + existing_defs["literal_or_identifier"]("addr"))("37:2")
         
-        asm_bcrsc = pyparsing.Group(pyparsing.Regex("BCRSC")("cmd") + existing_defs["literal_or_identifier"]("n_bit") + existing_defs["literal_or_identifier"]("addr"))("38:2")
-        asm_bcrss = pyparsing.Group(pyparsing.Regex("BCRSS")("cmd") + existing_defs["literal_or_identifier"]("n_bit") + existing_defs["literal_or_identifier"]("addr"))("39:2")
+        asm_bcrsc = pyparsing.Group(pyparsing.Regex("BCRSC|BTSTSC")("cmd") + existing_defs["literal_or_identifier"]("n_bit") + existing_defs["literal_or_identifier"]("addr"))("38:2")
+        asm_bcrss = pyparsing.Group(pyparsing.Regex("BCRSS|BTSTSS")("cmd") + existing_defs["literal_or_identifier"]("n_bit") + existing_defs["literal_or_identifier"]("addr"))("39:2")
         asm_jump = pyparsing.Group(pyparsing.Regex("JUMP")("cmd") + existing_defs["literal_or_identifier"]("addr"))("40:1")
         asm_jumpi = pyparsing.Group(pyparsing.Regex("JUMPI")("cmd") + existing_defs["literal_or_identifier"]("iaddr"))("41:1")
         asm_call = pyparsing.Group(pyparsing.Regex("CALL")("cmd") + existing_defs["literal_or_identifier"]("addr"))("42:1")
@@ -804,6 +892,9 @@ class Digirule2U(Digirule):
         asm_comout = pyparsing.Group(pyparsing.Regex("COMOUT")("cmd"))("192:0")
         asm_comin = pyparsing.Group(pyparsing.Regex("COMIN")("cmd"))("193:0")
         asm_comrdy = pyparsing.Group(pyparsing.Regex("COMRDY")("cmd"))("194:0")
+        asm_pinout = pyparsing.Group(pyparsing.Regex("PINOUT")("cmd") + existing_defs["literal_or_identifier"]("value"))("196:1")
+        asm_pinin = pyparsing.Group(pyparsing.Regex("PININ")("cmd") + existing_defs["literal_or_identifier"]("value"))("197:1")
+        asm_pindir = pyparsing.Group(pyparsing.Regex("PINDIR")("cmd") + existing_defs["literal_or_identifier"]("value"))("198:1")
         
         asm_statement = pyparsing.Group(asm_halt ^ asm_nop ^ asm_speed ^ asm_initsp ^ asm_copyla ^ asm_copylr ^ \
                                         asm_copyli ^ asm_copyar ^ asm_copyai ^ asm_copyra ^ asm_copyrr ^ asm_copyri ^ \
@@ -813,6 +904,7 @@ class Digirule2U(Digirule):
                                         asm_incr ^ asm_decrjz ^ asm_incrjz ^ asm_shiftrl ^ asm_shiftrr ^ asm_cbr ^ \
                                         asm_sbr ^ asm_bchg ^ asm_bcrsc ^ asm_bcrss ^ \
                                         asm_jump ^ asm_jumpi ^ asm_call ^ asm_calli ^ asm_return ^ asm_retla ^ \
-                                        asm_addrpc ^ asm_randa ^ asm_comout ^ asm_comin ^ asm_comrdy)
+                                        asm_addrpc ^ asm_randa ^ asm_comout ^ asm_comin ^ asm_comrdy ^ \
+                                        asm_pinout ^ asm_pinin ^ asm_pindir)
 
         return asm_statement
